/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Copyright (C) 2004 
 *
 * Authors: Erik Eliasson <eliasson@it.kth.se>
 *          Johan Bilien <jobi@via.ecp.fr>
*/

#ifndef MIKEYEXCEPTION_H
#define MIKEYEXCEPTION_H

/**
 * Base class for all exceptions that may occur in the MIKEY implementation.
 * @author Erik Eliasson, Johan Bilien
 * @version 0.01
 */

#include<libmikey/MikeyMessage.h>



class MikeyException{
	public:
		/**
		 * @param All exceptions MUST have a string describing the
		 * 	exception that is suitable to present to the user.
		 */
		MikeyException(string message);
		virtual ~MikeyException(){};

		/**
		 * @returns string describing the exception.
		 */
		virtual string message();
		
	private:
		string messageValue;

};


class MikeyExceptionUninitialized: public MikeyException{
	public:
		MikeyExceptionUninitialized(string msg);
		virtual ~MikeyExceptionUninitialized();

};



class MikeyExceptionMessageContent: public MikeyException{
	public:
		MikeyExceptionMessageContent(string msg);
		MikeyExceptionMessageContent(MikeyMessage * errMsg, string msg="");
		virtual ~MikeyExceptionMessageContent();

		MikeyMessage * errorMessage();
	private:
		MikeyMessage * errorMessageValue;

};



class MikeyExceptionMessageLengthException: public MikeyException{
	public:
		MikeyExceptionMessageLengthException(string msg);
		virtual ~MikeyExceptionMessageLengthException();

};


class MikeyExceptionNullPointerException : public MikeyException{
	public:
		MikeyExceptionNullPointerException(string msg);
		virtual ~MikeyExceptionNullPointerException();

};


class MikeyExceptionAuthentication : public MikeyException{
	public:
		MikeyExceptionAuthentication(string msg);
		virtual ~MikeyExceptionAuthentication();

};

class MikeyExceptionUnacceptable : public MikeyException{
	public:
		MikeyExceptionUnacceptable(string msg);
		virtual ~MikeyExceptionUnacceptable();

};

class MikeyExceptionUnimplemented : public MikeyException{
	public:
		MikeyExceptionUnimplemented(string msg);
		virtual ~MikeyExceptionUnimplemented();

};
#endif
