/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Copyright (C) 2004 
 *
 * Authors: Erik Eliasson <eliasson@it.kth.se>
 *          Johan Bilien <jobi@via.ecp.fr>
*/

#ifndef KEYAGREEMENT_H
#define KEYAGREEMENT_H
#include<assert.h>
#include<iostream>
#include<openssl/hmac.h>
#include<config.h>
#include<libmikey/MikeyDefs.h>
#include<libmikey/keyvalidity.h>

// different type of key derivation defined in MIKEY
#define KEY_DERIV_TEK          0
#define KEY_DERIV_SALT         1
#define KEY_DERIV_TRANS_ENCR   2
#define KEY_DERIV_TRANS_SALT   3
#define KEY_DERIV_TRANS_AUTH   4

#define KEY_AGREEMENT_TYPE_DH 	0
#define KEY_AGREEMENT_TYPE_PSK 	1
#define KEY_AGREEMENT_TYPE_PK 	2


class KeyAgreement{
	public:
		KeyAgreement();
		~KeyAgreement();

		/* Type of key agreement (DH, PSK, PKE) */
		int32_t type(){ return typeValue; };

		/* RAND value exchanged during the key agreement */
		unsigned int randLength();
		byte_t * rand();
		void setRand( byte_t * randData, int randLength );

		/* TEK and SALT values, derived from the TGK */
		void genTek( byte_t cs_id, unsigned int csb_id,
			     byte_t * tek, unsigned int tek_length );
		void genSalt( byte_t cs_id, unsigned int csb_id,
			      byte_t * salt, unsigned int salt_length );

		/* CSB ID: should be random in most cases and generated
		 * by the initiator */
		unsigned int csbId();
		virtual void setCsbId( unsigned int );

		/* TGK */
		void setTgk( byte_t * tgk, unsigned int tgkLength );
		unsigned int tgkLength();
		byte_t * tgk();

		/* KeyValidity information, exchanged during the key 
		 * agreement. NULL by default */
		KeyValidity * keyValidity();
		void setKeyValidity( KeyValidity * kv );

		/* Access the initiator and responder key agreement data
		 * (MIKEY messages when using MIKEY) */
		void * initiatorData();
		void setInitiatorData( void * );
		void * responderData();
		void setResponderData( void * );

		string authError();
		void setAuthError( string error );

	protected:
		void keyDeriv( byte_t cs_id, unsigned int csb_id,
		        	byte_t * inkey, unsigned int inkey_length,
		        	byte_t * key, unsigned int key_length,
			   	int type );
		byte_t * tgkPtr;
		unsigned int tgkLengthValue;
		byte_t * randPtr;
		unsigned int randLengthValue;

		unsigned int csbIdValue;
		int32_t typeValue;

		KeyValidity * kvPtr;

		void * initiatorDataPtr;
		void * responderDataPtr;

		string authErrorValue;
};

#endif
